@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-09 by the Quassel Project                          *
+ *   Copyright (C) 2005-2010 by the Quassel Project                        *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -60,6 +60,22 @@ QString stripFormatCodes(QString str) {
   return str;
 }
 
+QString stripAcceleratorMarkers(const QString &label_) {
+  QString label = label_;
+  int p = 0;
+  forever {
+    p = label.indexOf('&', p);
+    if(p < 0 || p + 1 >= label.length())
+      break;
+
+    if(label.at(p + 1).isLetterOrNumber() || label.at(p + 1) == '&')
+      label.remove(p, 1);
+
+    ++p;
+  }
+  return label;
+}
+
 QString decodeString(const QByteArray &input, QTextCodec *codec) {
   // First, we check if it's utf8. It is very improbable to encounter a string that looks like
   // valid utf8, but in fact is not. This means that if the input string passes as valid utf8, it
