@@ -26,6 +26,7 @@
 #  include <KMenuBar>
 #  include <KShortcutsDialog>
 #  include <KStatusBar>
+#  include <KToggleFullScreenAction>
 #  include <KToolBar>
 #  include <KWindowSystem>
 #endif
@@ -69,7 +70,6 @@
 #include "inputwidget.h"
 #include "irclistmodel.h"
 #include "ircconnectionwizard.h"
-#include "jumpkeyhandler.h"
 #include "legacysystemtray.h"
 #include "msgprocessorstatuswidget.h"
 #include "nicklistwidget.h"
@@ -120,6 +120,10 @@
 #include "settingspages/notificationssettingspage.h"
 #include "settingspages/topicwidgetsettingspage.h"
 
+#ifndef HAVE_KDE
+#  include "settingspages/shortcutssettingspage.h"
+#endif
+
 MainWin::MainWin(QWidget *parent)
 #ifdef HAVE_KDE
   : KMainWindow(parent),
@@ -131,7 +135,8 @@ MainWin::MainWin(QWidget *parent)
     _coreConnectionStatusWidget(new CoreConnectionStatusWidget(Client::coreConnection(), this)),
     _titleSetter(this),
     _awayLog(0),
-    _layoutLoaded(false)
+    _layoutLoaded(false),
+    _activeBufferViewIndex(-1)
 {
   setAttribute(Qt::WA_DeleteOnClose, false);  // we delete the mainwin manually
 
@@ -146,8 +151,6 @@ MainWin::MainWin(QWidget *parent)
   setWindowTitle("Quassel IRC");
   setWindowIconText("Quassel IRC");
   updateIcon();
-
-  installEventFilter(new JumpKeyHandler(this));
 }
 
 void MainWin::init() {
@@ -182,6 +185,7 @@ void MainWin::init() {
   setupNickWidget();
   setupInputWidget();
   setupChatMonitor();
+  setupViewMenuTail();
   setupStatusBar();
   setupToolBars();
   setupSystray();
@@ -306,7 +310,7 @@ void MainWin::updateIcon() {
 }
 
 void MainWin::setupActions() {
-  ActionCollection *coll = QtUi::actionCollection("General");
+  ActionCollection *coll = QtUi::actionCollection("General", tr("General"));
   // File
   coll->addAction("ConnectCore", new Action(SmallIcon("network-connect"), tr("&Connect to Core..."), coll,
                                              this, SLOT(showCoreConnectionDlg())));
@@ -338,7 +342,18 @@ void MainWin::setupActions() {
   coll->addAction("ToggleStatusBar", new Action(tr("Show Status &Bar"), coll,
                                                 0, 0))->setCheckable(true);
 
+#ifdef HAVE_KDE
+  QAction *fullScreenAct = KStandardAction::fullScreen(this, SLOT(toggleFullscreen()), this, coll);
+#else
+  QAction *fullScreenAct = new Action(SmallIcon("view-fullscreen"), tr("&Full Screen Mode"), coll,
+                             this, SLOT(toggleFullscreen()), QKeySequence(Qt::Key_F11));
+  fullScreenAct->setCheckable(true);
+#endif
+  coll->addAction("ToggleFullscreen", fullScreenAct);
+
   // Settings
+  coll->addAction("ConfigureShortcuts", new Action(SmallIcon("configure-shortcuts"), tr("Configure &Shortcuts..."), coll,
+                                                  this, SLOT(showShortcutsDlg())));
   coll->addAction("ConfigureQuassel", new Action(SmallIcon("configure"), tr("&Configure Quassel..."), coll,
                                                   this, SLOT(showSettingsDlg()), QKeySequence(Qt::Key_F7)));
 
@@ -361,8 +376,71 @@ void MainWin::setupActions() {
                                        QtUi::style(), SLOT(reload()), QKeySequence::Refresh));
 
   // Navigation
+  coll = QtUi::actionCollection("Navigation", tr("Navigation"));
+
   coll->addAction("JumpHotBuffer", new Action(tr("Jump to hot chat"), coll,
                                               this, SLOT(on_jumpHotBuffer_triggered()), QKeySequence(Qt::META + Qt::Key_A)));
+
+  // Jump keys
+#ifdef Q_WS_MAC
+  const int bindModifier = Qt::ControlModifier | Qt::AltModifier;
+  const int jumpModifier = Qt::ControlModifier;
+#else
+  const int bindModifier = Qt::ControlModifier;
+  const int jumpModifier = Qt::AltModifier;
+#endif
+
+  coll->addAction("BindJumpKey0", new Action(tr("Set Quick Access #0"), coll, this, SLOT(bindJumpKey()),
+                                             QKeySequence(bindModifier + Qt::Key_0)))->setProperty("Index", 0);
+  coll->addAction("BindJumpKey1", new Action(tr("Set Quick Access #1"), coll, this, SLOT(bindJumpKey()),
+                                             QKeySequence(bindModifier + Qt::Key_1)))->setProperty("Index", 1);
+  coll->addAction("BindJumpKey2", new Action(tr("Set Quick Access #2"), coll, this, SLOT(bindJumpKey()),
+                                             QKeySequence(bindModifier + Qt::Key_2)))->setProperty("Index", 2);
+  coll->addAction("BindJumpKey3", new Action(tr("Set Quick Access #3"), coll, this, SLOT(bindJumpKey()),
+                                             QKeySequence(bindModifier + Qt::Key_3)))->setProperty("Index", 3);
+  coll->addAction("BindJumpKey4", new Action(tr("Set Quick Access #4"), coll, this, SLOT(bindJumpKey()),
+                                             QKeySequence(bindModifier + Qt::Key_4)))->setProperty("Index", 4);
+  coll->addAction("BindJumpKey5", new Action(tr("Set Quick Access #5"), coll, this, SLOT(bindJumpKey()),
+                                             QKeySequence(bindModifier + Qt::Key_5)))->setProperty("Index", 5);
+  coll->addAction("BindJumpKey6", new Action(tr("Set Quick Access #6"), coll, this, SLOT(bindJumpKey()),
+                                             QKeySequence(bindModifier + Qt::Key_6)))->setProperty("Index", 6);
+  coll->addAction("BindJumpKey7", new Action(tr("Set Quick Access #7"), coll, this, SLOT(bindJumpKey()),
+                                             QKeySequence(bindModifier + Qt::Key_7)))->setProperty("Index", 7);
+  coll->addAction("BindJumpKey8", new Action(tr("Set Quick Access #8"), coll, this, SLOT(bindJumpKey()),
+                                             QKeySequence(bindModifier + Qt::Key_8)))->setProperty("Index", 8);
+  coll->addAction("BindJumpKey9", new Action(tr("Set Quick Access #9"), coll, this, SLOT(bindJumpKey()),
+                                             QKeySequence(bindModifier + Qt::Key_9)))->setProperty("Index", 9);
+
+  coll->addAction("JumpKey0", new Action(tr("Quick Access #0"), coll, this, SLOT(onJumpKey()),
+                                         QKeySequence(jumpModifier + Qt::Key_0)))->setProperty("Index", 0);
+  coll->addAction("JumpKey1", new Action(tr("Quick Access #1"), coll, this, SLOT(onJumpKey()),
+                                         QKeySequence(jumpModifier + Qt::Key_1)))->setProperty("Index", 1);
+  coll->addAction("JumpKey2", new Action(tr("Quick Access #2"), coll, this, SLOT(onJumpKey()),
+                                         QKeySequence(jumpModifier + Qt::Key_2)))->setProperty("Index", 2);
+  coll->addAction("JumpKey3", new Action(tr("Quick Access #3"), coll, this, SLOT(onJumpKey()),
+                                         QKeySequence(jumpModifier + Qt::Key_3)))->setProperty("Index", 3);
+  coll->addAction("JumpKey4", new Action(tr("Quick Access #4"), coll, this, SLOT(onJumpKey()),
+                                         QKeySequence(jumpModifier + Qt::Key_4)))->setProperty("Index", 4);
+  coll->addAction("JumpKey5", new Action(tr("Quick Access #5"), coll, this, SLOT(onJumpKey()),
+                                         QKeySequence(jumpModifier + Qt::Key_5)))->setProperty("Index", 5);
+  coll->addAction("JumpKey6", new Action(tr("Quick Access #6"), coll, this, SLOT(onJumpKey()),
+                                         QKeySequence(jumpModifier + Qt::Key_6)))->setProperty("Index", 6);
+  coll->addAction("JumpKey7", new Action(tr("Quick Access #7"), coll, this, SLOT(onJumpKey()),
+                                         QKeySequence(jumpModifier + Qt::Key_7)))->setProperty("Index", 7);
+  coll->addAction("JumpKey8", new Action(tr("Quick Access #8"), coll, this, SLOT(onJumpKey()),
+                                         QKeySequence(jumpModifier + Qt::Key_8)))->setProperty("Index", 8);
+  coll->addAction("JumpKey9", new Action(tr("Quick Access #9"), coll, this, SLOT(onJumpKey()),
+                                         QKeySequence(jumpModifier + Qt::Key_9)))->setProperty("Index", 9);
+
+  // Buffer navigation
+  coll->addAction("NextBufferView", new Action(SmallIcon("go-next-view"), tr("Activate Next Chat List"), coll,
+                                               this, SLOT(nextBufferView()), QKeySequence(QKeySequence::Forward)));
+  coll->addAction("PreviousBufferView", new Action(SmallIcon("go-previous-view"), tr("Activate Previous Chat List"), coll,
+                                                   this, SLOT(previousBufferView()), QKeySequence::Back));
+  coll->addAction("NextBuffer", new Action(SmallIcon("go-down"), tr("Go to Next Chat"), coll,
+                                           this, SLOT(nextBuffer()), QKeySequence(Qt::ALT + Qt::Key_Down)));
+  coll->addAction("PreviousBuffer", new Action(SmallIcon("go-up"), tr("Go to Previous Chat"), coll,
+                                               this, SLOT(previousBuffer()), QKeySequence(Qt::ALT + Qt::Key_Up)));
 }
 
 void MainWin::setupMenus() {
@@ -408,6 +486,8 @@ void MainWin::setupMenus() {
 #ifdef HAVE_KDE
   _settingsMenu->addAction(KStandardAction::configureNotifications(this, SLOT(showNotificationsDlg()), this));
   _settingsMenu->addAction(KStandardAction::keyBindings(this, SLOT(showShortcutsDlg()), this));
+#else
+  _settingsMenu->addAction(coll->action("ConfigureShortcuts"));
 #endif
   _settingsMenu->addAction(coll->action("ConfigureQuassel"));
 
@@ -472,7 +552,11 @@ void MainWin::addBufferView(ClientBufferViewConfig *config) {
   _bufferViewsMenu->addAction(dock->toggleViewAction());
 
   connect(dock->toggleViewAction(), SIGNAL(toggled(bool)), this, SLOT(bufferViewToggled(bool)));
+  connect(dock, SIGNAL(visibilityChanged(bool)), SLOT(bufferViewVisibilityChanged(bool)));
   _bufferViews.append(dock);
+
+  if(!activeBufferView())
+    nextBufferView();
 }
 
 void MainWin::removeBufferView(int bufferViewConfigId) {
@@ -486,8 +570,15 @@ void MainWin::removeBufferView(int bufferViewConfigId) {
     dock = qobject_cast<BufferViewDock *>(action->parent());
     if(dock && actionData.toInt() == bufferViewConfigId) {
       removeAction(action);
-      _bufferViews.removeAll(dock);
       Client::bufferViewOverlay()->removeView(dock->bufferViewId());
+      _bufferViews.removeAll(dock);
+
+      if(dock->isActive()) {
+        dock->setActive(false);
+        _activeBufferViewIndex = -1;
+        nextBufferView();
+      }
+
       dock->deleteLater();
     }
   }
@@ -508,11 +599,18 @@ void MainWin::bufferViewToggled(bool enabled) {
   if(!_bufferViews.contains(dock))
     return;
 
-  if(enabled) {
+  if(enabled)
     Client::bufferViewOverlay()->addView(dock->bufferViewId());
-  } else {
+  else
     Client::bufferViewOverlay()->removeView(dock->bufferViewId());
-  }
+}
+
+void MainWin::bufferViewVisibilityChanged(bool visible) {
+  Q_UNUSED(visible);
+  BufferViewDock *dock = qobject_cast<BufferViewDock *>(sender());
+  Q_ASSERT(dock);
+  if((!dock->isHidden() && !activeBufferView()) || (dock->isHidden() && dock->isActive()))
+    nextBufferView();
 }
 
 BufferView *MainWin::allBuffersView() const {
@@ -522,6 +620,84 @@ BufferView *MainWin::allBuffersView() const {
   return 0;
 }
 
+BufferView *MainWin::activeBufferView() const {
+  if(_activeBufferViewIndex < 0 || _activeBufferViewIndex >= _bufferViews.count())
+    return 0;
+  BufferViewDock *dock = _bufferViews.at(_activeBufferViewIndex);
+  return dock->isActive() ? qobject_cast<BufferView*>(dock->widget()) : 0;
+}
+
+void MainWin::changeActiveBufferView(int bufferViewId) {
+  if(bufferViewId < 0)
+    return;
+
+  BufferView *current = activeBufferView();
+  if(current) {
+    qobject_cast<BufferViewDock*>(current->parent())->setActive(false);
+    _activeBufferViewIndex = -1;
+  }
+
+  for(int i = 0; i < _bufferViews.count(); i++) {
+    BufferViewDock *dock = _bufferViews.at(i);
+    if(dock->bufferViewId() == bufferViewId && !dock->isHidden()) {
+      _activeBufferViewIndex = i;
+      dock->setActive(true);
+      return;
+    }
+  }
+
+  nextBufferView(); // fallback
+}
+
+void MainWin::changeActiveBufferView(bool backwards) {
+  BufferView *current = activeBufferView();
+  if(current)
+    qobject_cast<BufferViewDock*>(current->parent())->setActive(false);
+
+  if(!_bufferViews.count())
+    return;
+
+  int c = _bufferViews.count();
+  while(c--) { // yes, this will reactivate the current active one if all others fail
+    if(backwards) {
+      if(--_activeBufferViewIndex < 0)
+        _activeBufferViewIndex = _bufferViews.count()-1;
+    } else {
+      if(++_activeBufferViewIndex >= _bufferViews.count())
+        _activeBufferViewIndex = 0;
+    }
+
+    BufferViewDock *dock = _bufferViews.at(_activeBufferViewIndex);
+    if(dock->isHidden())
+      continue;
+
+    dock->setActive(true);
+    return;
+  }
+
+  _activeBufferViewIndex = -1;
+}
+
+void MainWin::nextBufferView() {
+  changeActiveBufferView(false);
+}
+
+void MainWin::previousBufferView() {
+  changeActiveBufferView(true);
+}
+
+void MainWin::nextBuffer() {
+  BufferView *view = activeBufferView();
+  if(view)
+    view->nextBuffer();
+}
+
+void MainWin::previousBuffer() {
+  BufferView *view = activeBufferView();
+  if(view)
+    view->previousBuffer();
+}
+
 void MainWin::showNotificationsDlg() {
   SettingsPageDlg dlg(new NotificationsSettingsPage(this), this);
   dlg.exec();
@@ -625,6 +801,11 @@ void MainWin::setupTopicWidget() {
   dock->toggleViewAction()->setText(tr("Show Topic Line"));
 }
 
+void MainWin::setupViewMenuTail() {
+  _viewMenu->addSeparator();
+  _viewMenu->addAction(QtUi::actionCollection("General")->action("ToggleFullscreen"));
+}
+
 void MainWin::setupTitleSetter() {
   _titleSetter.setModel(Client::bufferModel());
   _titleSetter.setSelectionModel(Client::bufferModel()->standardSelectionModel());
@@ -747,10 +928,13 @@ void MainWin::setConnectedState() {
     wizard->show();
   }
   else {
-    QtUiSettings s;
-    BufferId lastUsedBufferId(s.value("LastUsedBufferId").toInt());
-    if(lastUsedBufferId.isValid())
-      Client::bufferModel()->switchToBuffer(lastUsedBufferId);
+    // Monolithic always preselects last used buffer - Client only if the connection died
+    if(Client::coreConnection()->wasReconnect() || Quassel::runMode() == Quassel::Monolithic) {
+      QtUiSettings s;
+      BufferId lastUsedBufferId(s.value("LastUsedBufferId").toInt());
+      if(lastUsedBufferId.isValid())
+        Client::bufferModel()->switchToBuffer(lastUsedBufferId);
+    }
   }
 }
 
@@ -766,14 +950,21 @@ void MainWin::loadLayout() {
   }
 
   restoreState(state, accountId);
+  int bufferViewId = s.value(QString("ActiveBufferView-%1").arg(accountId), -1).toInt();
+  if(bufferViewId >= 0)
+    changeActiveBufferView(bufferViewId);
+
   _layoutLoaded = true;
 }
 
 void MainWin::saveLayout() {
   QtUiSettings s;
   int accountId = _bufferViews.count()? Client::currentCoreAccount().accountId().toInt() : 0; // only save if we still have a layout!
-  if(accountId > 0)
+  if(accountId > 0) {
     s.setValue(QString("MainWinState-%1").arg(accountId) , saveState(accountId));
+    BufferView *view = activeBufferView();
+    s.setValue(QString("ActiveBufferView-%1").arg(accountId), view ? view->config()->bufferViewId() : -1);
+  }
 }
 
 void MainWin::disconnectedFromCore() {
@@ -796,7 +987,14 @@ void MainWin::disconnectedFromCore() {
     }
   }
 
+  // store last active buffer
   QtUiSettings s;
+  BufferId lastBufId = _bufferWidget->currentBuffer();
+  if(lastBufId.isValid()) {
+    s.setValue("LastUsedBufferId", lastBufId.toInt());
+    // clear the current selection
+    Client::bufferModel()->standardSelectionModel()->clearSelection();
+  }
   restoreState(s.value("MainWinState").toByteArray());
   setDisconnectedState();
 }
@@ -974,11 +1172,24 @@ void MainWin::showAboutDlg() {
   AboutDlg(this).exec();
 }
 
-#ifdef HAVE_KDE
 void MainWin::showShortcutsDlg() {
-  KShortcutsDialog::configure(QtUi::actionCollection("General"), KShortcutsEditor::LetterShortcutsDisallowed);
-}
+#ifdef HAVE_KDE
+  KShortcutsDialog dlg(KShortcutsEditor::AllActions, KShortcutsEditor::LetterShortcutsDisallowed, this);
+  foreach(KActionCollection *coll, QtUi::actionCollections())
+    dlg.addCollection(coll, coll->property("Category").toString());
+  dlg.exec();
+#else
+  SettingsPageDlg dlg(new ShortcutsSettingsPage(QtUi::actionCollections(), this), this);
+  dlg.exec();
 #endif
+}
+
+void MainWin::toggleFullscreen() {
+  if(isFullScreen())
+    showNormal();
+  else
+    showFullScreen();
+}
 
 /********************************************************************************************************/
 
@@ -1101,6 +1312,14 @@ void MainWin::clientNetworkUpdated() {
   switch(net->connectionState()) {
   case Network::Initialized:
     action->setIcon(SmallIcon("network-connect"));
+    // if we have no currently selected buffer, jump to the first connecting statusbuffer
+    if(!bufferWidget()->currentBuffer().isValid()) {
+      QModelIndex idx = Client::networkModel()->networkIndex(net->networkId());
+      if(idx.isValid()) {
+        BufferId statusBufferId = idx.data(NetworkModel::BufferIdRole).value<BufferId>();
+        Client::bufferModel()->switchToBuffer(statusBufferId);
+      }
+    }
     break;
   case Network::Disconnected:
     action->setIcon(SmallIcon("network-disconnect"));
@@ -1136,6 +1355,34 @@ void MainWin::on_jumpHotBuffer_triggered() {
   Client::bufferModel()->switchToBuffer(bufferId);
 }
 
+void MainWin::onJumpKey() {
+  QAction *action = qobject_cast<QAction *>(sender());
+  if(!action || !Client::bufferModel())
+    return;
+  int idx = action->property("Index").toInt();
+
+  if(_jumpKeyMap.isEmpty())
+    _jumpKeyMap = CoreAccountSettings().jumpKeyMap();
+
+  if(!_jumpKeyMap.contains(idx))
+    return;
+
+  BufferId buffer = _jumpKeyMap.value(idx);
+  if(buffer.isValid())
+    Client::bufferModel()->switchToBuffer(buffer);
+
+}
+
+void MainWin::bindJumpKey() {
+  QAction *action = qobject_cast<QAction *>(sender());
+  if(!action || !Client::bufferModel())
+    return;
+  int idx = action->property("Index").toInt();
+
+  _jumpKeyMap[idx] = Client::bufferModel()->currentBuffer();
+  CoreAccountSettings().setJumpKeyMap(_jumpKeyMap);
+}
+
 void MainWin::on_actionDebugNetworkModel_triggered() {
   QTreeView *view = new QTreeView;
   view->setAttribute(Qt::WA_DeleteOnClose);
